from pathlib import Path

from rich.console import Console
from rich.panel import Panel

from ..utils import read, write

console = Console(width=88)


################################################################
# init package
################################################################
def init_settings(root, verbose=False):
    # get project name from input
    fp = Path(root)
    project = fp.absolute().name

    # [WRITE FILES]
    overwritten = list()
    for FILE in [".gitignore", ".dockerignore", ".vscode/settings.json"]:
        _fp = fp / FILE
        confirm = True
        if _fp.exists():
            answer = input(f"'{_fp}' is exists. overwrite? (y/N) ")
            if answer.strip().lower() != "y":
                confirm = False
            else:
                overwritten += [FILE]
        if confirm:
            write(dst_fp=fp / FILE, content=read(FILE))

    # [LOGGING]
    if verbose:
        summary = [f"[bold]Settings for Project '{project}' is Ready.[/bold]"]
        if overwritten:
            summary += [" - [NOTE] " + ", ".join(overwritten) + " are overwritten."]
        console.print(Panel("\n".join(summary)))



import os
from pathlib import Path

from rich.console import Console
from rich.panel import Panel

from ..exceptions import CancelInit
from ..utils import STATIC_DIR, read, write

console = Console(width=88)


################################################################
# init package
################################################################
def init_package(root):
    from InquirerPy import prompt

    # get project name from input
    fp = Path(root)
    project = fp.absolute().name

    # get private pypi url
    private_pypi = os.getenv("PIP_INDEX_URL") or ""

    # set setup configuration
    setup_conf = [
        {"type": "input", "name": "project", "default": project, "message": "Project Name"},
        {"type": "input", "name": "author", "message": "Author"},
        {"type": "input", "name": "author_email", "message": "Author Email"},
        {"type": "input", "name": "description", "message": "Description"},
        {"type": "input", "name": "long_description", "message": "Long Description"},
        {"type": "input", "name": "keywords", "message": "Keywords"},
        {"type": "input", "name": "license", "default": "MIT License", "message": "License"},
        {"type": "input", "name": "python_requires", "default": ">= 3.9", "message": "Python Requires"},
        {
            "type": "input",
            "name": "install_requires",
            "message": "Install Requires (e.g., pandas, numpy)",
            "filter": lambda x: [_.strip() for _ in x.split(",")],
        },
        {
            "type": "input",
            "name": "dependency_links",
            "default": private_pypi,
            "message": "Your Private PyPI",
        },
        {"type": "confirm", "name": "confirm", "message": "Confirm"},
    ]

    # user input
    conf = prompt(setup_conf)
    confirm = conf.pop("confirm")
    if not confirm:
        raise CancelInit

    # correct install requires
    if conf["install_requires"] is not None:
        TAB = "    "
        for default_package in ["click"]:
            if default_package not in conf["install_requires"]:
                conf["install_requires"].insert(0, default_package)
        conf["install_requires"] = f"\n{TAB}".join(conf["install_requires"])

    # update package_name (replace "-" to "_" from project)
    conf.update({"package_name": conf["project"].replace("-", "_")})

    # [WRITE FILES]
    # MANIFEST.in
    MANIFEST_IN_FILE = "MANIFEST.in"
    write(
        dst_fp=fp / MANIFEST_IN_FILE,
        content=read(MANIFEST_IN_FILE).format(**conf),
    )

    # pyproject.toml
    PYPROJECT_TOML_FILE = "pyproject.toml"
    write(
        dst_fp=fp / PYPROJECT_TOML_FILE,
        content=read(PYPROJECT_TOML_FILE).format(**conf),
    )

    # setup.cfg
    SETUP_CFG_FILE = "setup.cfg"
    write(
        dst_fp=fp / SETUP_CFG_FILE,
        content=read(SETUP_CFG_FILE).format(**conf),
    )

    # Dockerfile
    DOCKERFILE_FILE = "Dockerfile"
    write(
        dst_fp=fp / DOCKERFILE_FILE,
        content=read(DOCKERFILE_FILE),
    )

    # make src directory
    (fp / "src").mkdir(exist_ok=True)
    package_name = conf["package_name"]
    package_root = fp / "src" / package_name
    package_root.mkdir(exist_ok=True)

    # src/__init__.py
    INIT_PY_FILE = "__init__.py"
    write(
        dst_fp=package_root / INIT_PY_FILE,
        content=read(INIT_PY_FILE).format(**conf),
    )

    # create static dir
    static_dir = package_root / "static"
    static_dir.mkdir(exist_ok=True)

    # example scripts: scripts.py
    SCRIPTS_PY = "scripts.py"
    write(dst_fp=package_root / SCRIPTS_PY, content=read(SCRIPTS_PY).format(**conf))

    # example.py for example scripts
    EXAMPLE_PY = "example.py"
    write(dst_fp=package_root / EXAMPLE_PY, content=read(EXAMPLE_PY).format(**conf))

    # [LOGGING]
    summary = "\n".join(
        [
            f"[bold]Python project '{project}' is ready.[/bold]",
            " - Build Package: 'python -m build', python package 'build' is required.",
            " - Upload to PyPI: 'twine upload dist/*', python package 'twine is required.",
        ]
    )
    print()
    console.print(Panel(summary))



from __future__ import print_function, unicode_literals

import click

from .exceptions import CancelInit


################################################################
# CLIs
################################################################
@click.group()
def bibigo():
    pass


################
# init
################
@bibigo.group()
def init():
    pass


@init.command()
@click.argument("root")
def settings(root):
    from .init.settings import init_settings

    try:
        init_settings(root, verbose=True)
    except CancelInit as ex:
        print("[EXIT] Nothing Happened!")
        return


@init.command()
@click.argument("root")
def package(root):
    from .init.package import init_package
    from .init.settings import init_settings

    try:
        init_package(root)
        init_settings(root)
    except CancelInit as ex:
        print("[EXIT] Nothing Happened!")
        return


@init.command()
@click.argument("root")
def docker_compose(root):
    from .init.docker_compose import init_stack

    try:
        init_stack(root)
    except CancelInit as ex:
        print("[EXIT] Nothing Happened!")
        return


################
# debug
################
@bibigo.group()
def debug():
    pass


@debug.command()
@click.argument("filename")
@click.option("--version", default="default")
def file(filename, version):
    from .utils import read

    content = read(fn=filename, version=version)
    print(content)



import json
import pkgutil
from pathlib import Path
from typing import Union

STATIC_DIR = "static"


# ensure path
def ensure_path(fp: Union[str, Path]):
    return Path(fp)


# read file
def read(fn: Path, version: str = "default"):
    fp = Path(STATIC_DIR) / f"{fn}.{version}"
    content = pkgutil.get_data(__name__, fp.as_posix()).decode()
    return content


# make dir if not exist and write file
def write(dst_fp: Path, content, mode="w"):
    dst_fp = ensure_path(dst_fp)

    # make directory
    dst_fp.parent.mkdir(parents=True, exist_ok=True)

    # write file
    with open(dst_fp, mode) as fw:
        if dst_fp.suffix.lower() in ["json"]:
            json.dump(content, fw)
        else:
            fw.write(content)



import os
from pathlib import Path

from rich.console import Console
from rich.panel import Panel

from ..exceptions import CancelInit
from ..utils import read, write

console = Console(width=88)


################################################################
# init package
################################################################
def init_stack(root):
    from InquirerPy import prompt

    # get service name from input
    fp = Path(root)
    service = fp.absolute().name

    # get private pypi url
    domain = os.getenv("DOMAIN") or "example.com"

    # set setup configuration
    deploy_conf = [
        {"type": "input", "name": "service", "default": service, "message": "Service Name"},
        {"type": "input", "name": "image", "default": "hello-world:latest", "message": "Image"},
        {"type": "input", "name": "command", "default": "/hello", "message": "Container Command"},
        {
            "type": "input",
            "name": "mode",
            "default": "replicated",
            "message": "Deploy Mode ('replicated' or 'global')",
        },
        {"type": "input", "name": "role", "default": "worker", "message": "Deploy Placement ('manager' or 'worker')"},
        {"type": "input", "name": "replicas", "default": "1", "message": "Replicas (No. of Containers)"},
        {"type": "input", "name": "http-entrypoint", "default": "web", "message": "Traefik Entrypoint for HTTP"},
        {
            "type": "input",
            "name": "https-entrypoint",
            "default": "websecure",
            "message": "Traefik Entrypoint for HTTPS",
        },
        {
            "type": "input",
            "name": "http-redirect",
            "default": "http-redirect",
            "message": "Traefik Middlware for HTTP Redirect",
        },
        {"type": "input", "name": "certresolver", "default": "lego", "message": "Certificate Resolver"},
        {"type": "input", "name": "traefik-network", "default": "traefik", "message": "Docker Network for Traefik"},
        {"type": "input", "name": "domain", "default": f"{service}.{domain}", "message": "Domain for Your Service"},
        {"type": "input", "name": "port", "default": "3000", "message": "Service Port"},
        {"type": "confirm", "name": "confirm", "message": "Confirm"},
    ]

    # user input
    conf = prompt(deploy_conf)
    confirm = conf.pop("confirm")
    if not confirm:
        raise CancelInit

    # [WRITE FILES]
    DOCKER_COMPOSE_FILE = "docker-compose.yml"
    write(dst_fp=fp / DOCKER_COMPOSE_FILE, content=read(DOCKER_COMPOSE_FILE).format(**conf))

    # [LOGGING]
    summary = "\n".join(
        [
            f"[bold]docker-compose.yml for '{service}' is ready.[/bold]",
        ]
    )
    print()
    console.print(Panel(summary))
